C*****************************************
       program filter twins
C*****************************************
       implicit none
	   include 'common.fi'

       parameter (CD=9)
       parameter (CD2=9)
       dimension gbd_f(CD,CD,CD,CD2,4*CD2)
       dimension gbd_o(CD,CD,CD,CD2,4*CD2)
       dimension e1(3), e2(3)
       dimension gg1(3,3), gg2(3,3), dg(3,3)
       dimension so1(3,3), so2(3,3), sa(2), so2t(3,3)

       write(6,*) '=============================================='
	   write(6,*) ' PROGRAM USED TO ANALYZE THE TWIN CONTENT'
	   write(6,*) ' OF LINE SEGMENT DATA. IT DETERMINES THE '
	   write(6,*) ' SIGMA THREE CONTENT AND ANALYZES THE  '
	   write(6,*) ' COHERENCY OF THESE BOUNDARIES.  '
	   write(6,*) ' IT ALSO CREATES NEW FILES WITH SEGMENTS '
	   write(6,*) ' CLASSIFIED AS SIGMA 3 AND NON-SIGMA 3 '
	   write(6,*)
	   write(6,*) ' DEVELOPED BY THE CMU MRSEC'
	   write(6,*) ' rohrer@cmu.edu'	   
	   write(6,*) ' version 01/25/2009'
       write(6,*) '=============================================='
	   
	   version = 'version 01/25/09'	   
	   
       pi = 4.0*atan(1.0)
	   eps = 1.e-6
	   HCD = 90.0
	   len = 0.0
	   lentot = 0.0
	   len1 = 0.0
	   len2 = 0.0
	   len3 = 0.0

c  The section reads information from the input file, including the data file name,
c  the symmetry type.

       open(21, file='input.txt', status='old')
	   read(21,*)
	   read(21,*) keyword
	   read(21,*) tempword, tempword, tempword
       read(21,*) msym, rotation, radian	   
	   	   
 10     format(A,A)
	    write(6,10)'I am going to look for data in the file labeled: ', keyword
 11     format(A,I2,2X,A,I2,2X,A,I2)
	    write(6,11)'msym =',msym,'rotation =',rotation,'radian =',radian
	   
	    close(21)

c  The statement calls the subroutine that contains the symmetry operators.  The variable
c  msym, read from the input file, specifies the symmetry (1 = tetragonal, 2 = hexagonal, 
c  3 = subic, 4 = trigonal), O is a matrix containing the operators, and nsymm is the 
c  number of symmetry operators.  The routine returns O and nsymm.

       call get_symop (msym, O, nsymm)

c This program is set up to sort observed misorientations by comparing them to a preset one.
c The preset misorientation is defined by the vector mis_ax(3) and the rotation by the paramter
c theta_mis.  The vector plane(3) can be used to define a particular grain boundary plane.
c The default is a sigma three boundary.

		mis_ax(1)=1/(3)**0.5
		mis_ax(2)=1/(3)**0.5
		mis_ax(3)=1/(3)**0.5
		theta_mis=60.0*(pi/180)
        plane(1)=1/(3)**0.5
        plane(2)=1/(3)**0.5
        plane(3)=1/(3)**0.5
		
		do i=1,100
		 diffs(i)=0
		enddo
		
c  To begin, we need to know the minimum misorientation of the reference
c  misorientation.

c         in the initial part, we transform the axis angle description to
c         a matrix, and assume a fictive second crystal has the identity 
c         orientation wrt the sample frame
		  call AAtoG(mis_ax, theta_mis, g1)
		  e2(1)=0.0
		  e2(2)=0.0
		  e2(3)=0.0
		  call EToG(e2,g2)
c         this indetifies the misorientation that has both the minimum
c         rotation angle and whose axis has three positive components.
c         This is taken as the reference misorietnation in the fundametnal zone.		  
		   disor1 = pi/2
           do i_s=1,nsymm	
		    call symop(O,i_s,so1)
		    call MToM (so1,g1,gg1)
			do j_s=1,nsymm
		     call symop(O,j_s,so2)
		     call MToM(so2,g2,gg2)
		     call trans(gg2,gt)
		     call MToM(gg1,gt,dg)
			   misor=acos((dg(1,1)+dg(2,2)+dg(3,3)-1)/2)
			   x(1)=(dg(2,3)-dg(3,2))/(2*sin(misor))
			   x(2)=(dg(3,1)-dg(1,3))/(2*sin(misor))
			   x(3)=(dg(1,2)-dg(2,1))/(2*sin(misor))
			   if (x(1).gt.0.0.and.x(2).gt.0.0.and.x(3).gt.0.0) then
			    if (misor.lt.disor1) then
				dis_g1=dg
				disor1=misor
			   endif
			  endif 	
		   call trans (gg1, gt)		   
		   call MToM (gg2, gt, dg)
               misor=acos((dg(1,1)+dg(2,2)+dg(3,3)-1)/2)
			   x(1)=(dg(2,3)-dg(3,2))/(2*sin(misor))
			   x(2)=(dg(3,1)-dg(1,3))/(2*sin(misor))
			   x(3)=(dg(1,2)-dg(2,1))/(2*sin(misor))
			   if (x(1).gt.0.0.and.x(2).gt.0.0.and.x(3).gt.0.0) then
			    if (misor.lt.disor1) then
				dis_g1=dg
				disor1=misor
			   endif
			  endif 
           enddo
		  enddo
		  
          misor=acos((dis_g1(1,1)+dis_g1(2,2)+dis_g1(3,3)-1)/2)
		  x(1)=(dis_g1(2,3)-dis_g1(3,2))/(2*sin(misor))
		  x(2)=(dis_g1(3,1)-dis_g1(1,3))/(2*sin(misor))
		  x(3)=(dis_g1(1,2)-dis_g1(2,1))/(2*sin(misor))
		  write(6,*) x(1),x(2),x(3),misor

c  This part is to determines the total number of line segments (lines in the data file),
c  assigned to the variable, nnline
		
		ct1=0
		ct2=0
		ct3=0
		
		nnline = 0
		open(41, file=keyword, status='old')
 60     continue
		read(41,*,end=61)
		nnline=nnline+1
		goto 60
 61     continue
        close(41)

 72		format(A,I7,A)
		write(6,72) 'There are',nnline,' lines in the data file.'
		
c Check each line for the string '#'. If found, we know this line is part  
c of the header.  If not, we have reached the data section.		
	   open(41, file=keyword, status='old')
       header = 0
       hash = 0
	   do i=1,nnline
	    read(41, "(a)") inline
	    hash = index(inline, '#')
	    if (hash.ne.0) then
	     header=header+1
		 hash=0
	    else
		 goto 80     
	    endif			
	   enddo

 80    close(41)				
	   write(6,*)header,' lines in the header'	
	
	   open(33, file='sig3_out.txt', status='unknown')	
	   open(34, file='not_sig3_out.txt', status='unknown')
		
		
		open(41, file=keyword, status='old')
		nline = 0
c       read the header
         do i=1,header
	      read(41, "(a)") inline
		  write(33, "(a)")inline
		  write(34, "(a)")inline
	     enddo	
		 
 83      format(A,A)    
		 write(33,"(a)")'# list of segments associated with sigma 3 grain boundaries'
         write(34,"(a)")'# list of segments associated with non-sigma 3 grain boundaries'
		 write(33,83)'# extracted from file ',keyword
		 write(34,83)'# extracted from file ',keyword
		 write(33,83)'# written by twin_filter ',version
		 write(34,83)'# written by twin_filter ',version	 

		
 117     format(7f9.3,f8.2,4f11.2)	
		
		do kk=header+1,nnline
c		write(6,*)kk
c  This section reads a single line from the data file
		
		read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	temp, temp, x1, y1, x2, y2
	
c  This part converts angular data to radians, if specified in the input file
c  by the variable, radian
				
	    if (radian.eq.0) then
	     e1(1)=e1(1)*(pi/180.0)
	     e1(2)=e1(2)*(pi/180.0)
	     e1(3)=e1(3)*(pi/180.0)

	     e2(1)=e2(1)*(pi/180.0)
	     e2(2)=e2(2)*(pi/180.0)
	     e2(3)=e2(3)*(pi/180.0)
	    endif

c  If the data are in the HKL reference frame, and the system is hexagonal,
c  then the data are rotated by -30 so that [2-1-10] is parallel to +x in 
c  the output (this direction is to the right in the plane of the paper)
		
	    if (rotation.eq.0.and.msym.eq.2) then
	     if (e1(3).gt.(30*pi/180)) then
 	      e1(3) = e1(3)-(30*pi/180)
		 else
		  e1(3) = e1(3)+(330*pi/180)
		 endif
		 if (e2(3).gt.(30*pi/180)) then
 	      e2(3) = e2(3)-(30*pi/180)
		 else
		  e2(3) = e2(3)+(330*pi/180)
		 endif
		endif
		
c  This part converts the Euler angles from the input file to a g matrix

        call EToG(e1,g1)
		call EToG(e2,g2)

c This section determines the length of the grain boundary segment, and its
c orientation.  The transformations make the [100] direction in the input file
c consistant with the definition in the result.  in the final stereograms, the
c [100] direction is in the plane of the paper, pointing to the right.


	    if (rotation.eq.1) then
	       nl(1)=y1-y2
	       nl(2)=x1-x2
		endif   
	    if (rotation.eq.0) then
	       nl(1)=x2-x1
	       nl(2)=-(y2-y1)
		endif
	    if (rotation.eq.2) then
	       nl(1)=x2-x1
	       nl(2)=y2-y1
		endif		

        nl(3)=0.0
		line=nl
		
	    templ=nl(2)
  	    nl(2)=nl(1)
	    nl(1)=-templ
	    nl(3)=0.0

	    length=sqrt(nl(1)*nl(1)+nl(2)*nl(2))
	    if (nl(1).ne.0.0) then
	       p=atan(nl(2)/nl(1))
	    else
	       p=pi*0.5
	    endif
	    p=atan2(nl(2),nl(1))

	    if (p.lt.0.0) p=pi+p
		
		lentot=lentot+length

c          to compare this to our reference misorientation, we need to
c          know the minimum misorientation for this observation.
c          this should work just like the section obove that finds the
c          mimimum misorientation for the reference direction.  In what 
c          this is taken as the reference frame.

           disor2 = pi/2
           do i_s=1,nsymm	
		    call symop(O,i_s,so1)
		    call MToM (so1,g1,gg1)
			do j_s=1,nsymm
		     call symop(O,j_s,so2)
		     call MToM(so2,g2,gg2)
		     call trans(gg2,gt)
		     call MToM(gg1,gt,dg)
			   misor=acos((dg(1,1)+dg(2,2)+dg(3,3)-1)/2)
			   x(1)=(dg(2,3)-dg(3,2))/(2*sin(misor))
			   x(2)=(dg(3,1)-dg(1,3))/(2*sin(misor))
			   x(3)=(dg(1,2)-dg(2,1))/(2*sin(misor))
			   if (x(1).gt.0.0.and.x(2).gt.0.0.and.x(3).gt.0.0) then
			    if (misor.lt.disor2) then
				dis_g2=dg
				gref1=gg1
				gref2=gg2
				xref=x
				disor2=misor
			   endif
			  endif 	
		   call trans (gg1, gt)		   
		   call MToM (gg2, gt, dg)
               misor=acos((dg(1,1)+dg(2,2)+dg(3,3)-1)/2)
			   x(1)=(dg(2,3)-dg(3,2))/(2*sin(misor))
			   x(2)=(dg(3,1)-dg(1,3))/(2*sin(misor))
			   x(3)=(dg(1,2)-dg(2,1))/(2*sin(misor))
			   if (x(1).gt.0.0.and.x(2).gt.0.0.and.x(3).gt.0.0) then
			    if (misor.lt.disor2) then
				dis_g2=dg
				gref1=gg1
				gref2=gg2
				xref=x				
				disor2=misor
			   endif
			  endif 
           enddo
		  enddo

c        with the representations of the current misorientation
c        and the reference misorientation both in the fundamental zone,
c        we can calculate the misorientation between them and be certain
c        that this is also a minimal misorientation.  That value is used
c        to determine if the observation is a sigma three.        
		  
		  call trans(dis_g1,gt)
		  call MToM(dis_g2,gt,dg)
		  misor=acos((dg(1,1)+dg(2,2)+dg(3,3)-1)/2)
			   x(1)=(dg(2,3)-dg(3,2))/(2*sin(misor))
			   x(2)=(dg(3,1)-dg(1,3))/(2*sin(misor))
			   x(3)=(dg(1,2)-dg(2,1))/(2*sin(misor))

c         If the misorientation between the observation and the reference
c         misorientation is less than Brandon's criterion, we count it as
c         a sigma three grain boundary.

          if (misor.lt.0.151) then
		   ct2=ct2+1
		   len2=len2+length
		   write(33,117)  e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	temp, temp, x1, y1, x2, y2

c         now we check to see if the line segment has an orientation that
c         is consistent with the twin plane.  In this case, we see if it is
c         oriented within ten degrees of being perpendicular to the common
c         axis of misorientation or to the ideal direction specified by plane.
c         Here, this is [111].  (use xref for the misorientation axis condition). 
c         The results are expected to be different for the different conditions.

		   call MToV (gref1, line, n1)
		   length1=((n1(1)**2)+(n1(2)**2)+(n1(3)**2))**0.5
c		   length2=((xref(1)**2)+(xref(2)**2)+(xref(3)**2))**0.5
c		   numer=(n1(1)*xref(1))+(n1(2)*xref(1))+(n1(3)*xref(1))
		   length2=((plane(1)**2)+(plane(2)**2)+(plane(3)**2))**0.5
		   numer=(n1(1)*plane(1))+(n1(2)*plane(1))+(n1(3)*plane(1))

		   
c          if the angle between the line segment and the common axis is within 
c          10 degrees of 90, then the ABS of the COS is less than 0.1736	
	   
		    theta=ABS(numer/length1*length2)
           	ns=int(acos(theta)*(180/pi))+1
		    diffs(ns)=diffs(ns)+1
		    if (theta.lt.0.1736) then

c		   to pass the test, the same must be true in the reference frame of the
c          second crystal.

		   call MToV (gref2, line, n2)
		   length1=((n2(1)**2)+(n2(2)**2)+(n2(3)**2))**0.5
c		   length2=((xref(1)**2)+(xref(2)**2)+(xref(3)**2))**0.5
c		   numer=(n2(1)*xref(1))+(n2(2)*xref(1))+(n2(3)*xref(1))
		   length2=((plane(1)**2)+(plane(2)**2)+(plane(3)**2))**0.5
		   numer=(n2(1)*plane(1))+(n2(2)*plane(1))+(n2(3)*plane(1))	   
		   theta=ABS(numer/length1*length2)

		   if (theta.lt.0.1736) then
		     ct3=ct3+1
			 len3=len3+length
           endif
		  else
		   goto 4000
		  endif 
		  
 4000     continue		   
		   
	      else
		   ct1=ct1+1
		   len1=len1+length
		   write(34,117)  e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), 
     &	temp, temp, x1, y1, x2, y2
		  endif
		  
c		  if (kk.eq.10000) goto 5100
		  
		  
		  
 5000	  enddo
 5050     format(A,X,I6)
 5060     format(A,X,I2,A)
 5061     format(A,X,F5.2,A)
 5070     format(A)
c    	  write(6,*),ct1,ct2,ct3
 5100     write(6,5050)'Non-Sigma 3 boundaries:',ct1
    	  write(6,5050)'Sigma 3 boundaries:',ct2
		  write(6,5050)'Coherent Sigma 3 boundaries:',ct3
		  write(6,*)' '
		  write(6,5060)'Non-Sigma 3 boundaries comprise',100*ct1/(ct1+ct2),
     &	  	  '% of the population'
		  write(6,5060)'Sigma 3 boundaries comprise',100*ct2/(ct1+ct2),
     &	  	  '% of the population'
 		  write(6,5060)'coherent Sigma 3 boundaries comprise',100*ct3/(ct1+ct2),
     &	  	  '% of the population'
		  write(6,*)' '	 
		  write(6,5061)'Non-Sigma 3 boundaries are',100*(len1/lentot),
     &	  	  '% of the length'	
		  write(6,5061)'Sigma 3 boundaries are',100*(len2/lentot),
     &	  	  '% of the length'	 
		  write(6,5061)'Coherent Sigma 3 boundaries are',100*(len3/lentot),
     &	  	  '% of the length'	

		  
          open(30, file='report.txt', status='unknown')
		  write(30,83)'based on file ',keyword
		  write(30,83)'written by twin_filter ',version
 
          write(30,5050)'Non-Sigma 3 boundaries:',ct1
    	  write(30,5050)'Sigma 3 boundaries:',ct2
		  write(30,5050)'Coherent Sigma 3 boundaries:',ct3
		  write(30,*)' '
		  write(30,5060)'Non-Sigma 3 boundaries comprise',100*ct1/(ct1+ct2),
     &	  	  '% of the population'
		  write(30,5060)'Sigma 3 boundaries comprise',100*ct2/(ct1+ct2),
     &	  	  '% of the population'
 		  write(30,5060)'coherent Sigma 3 boundaries comprise',100*ct3/(ct1+ct2),
     &	  	  '% of the population'
		  write(30,*)' '	 
		  write(30,5061)'Non-Sigma 3 boundaries are',100*(len1/lentot),
     &	  	  '% of the length'	
		  write(30,5061)'Sigma 3 boundaries are',100*(len2/lentot),
     &	  	  '% of the length'	 
		  write(30,5061)'Coherent Sigma 3 boundaries are',100*(len3/lentot),
     &	  	  '% of the length'	
	 
		  write(30,*)' '
		  write(30,5070)'differ number' 
          
 
 
 
 5200     format(I2,5x,I5)
          do i=1,90
          write(30,5200)i, diffs(91-i)
		  enddo

          close(30)
		  close(33)
		  close(34)

       end